#
# raccp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2.
#

require 'racc/parser'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @yydebug     = racc.d_parse && Racc_debug_parser
      @ruletable   = racc.ruletable
      @symboltable = racc.symboltable

      @class_name = nil
      @super_class = 'Racc::Parser'
    end

    attr :class_name
    attr :super_class

    def parse( str )
      @scanner = RaccScanner.new( str )
      @scanner.debug = @yydebug

      do_parse
    end


    private

    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      if val.respond_to? :id2name then
        v = val.id2name
      elsif String === val then
        v = val
      else
        v = val.inspect
      end
      raise ParseError, "#{@scanner.lineno}: unexpected token '#{v}'"
    end


##### racc 1.2.5 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 6, 23, :_reduce_1,
 1, 24, :_reduce_2,
 3, 24, :_reduce_3,
 0, 25, :_reduce_none,
 2, 25, :_reduce_none,
 3, 27, :_reduce_6,
 1, 27, :_reduce_none,
 2, 27, :_reduce_8,
 2, 27, :_reduce_9,
 2, 27, :_reduce_10,
 2, 28, :_reduce_11,
 3, 28, :_reduce_12,
 3, 29, :_reduce_13,
 3, 29, :_reduce_14,
 1, 33, :_reduce_none,
 2, 33, :_reduce_none,
 2, 34, :_reduce_17,
 2, 34, :_reduce_18,
 2, 34, :_reduce_19,
 1, 31, :_reduce_20,
 2, 31, :_reduce_21,
 2, 31, :_reduce_none,
 1, 30, :_reduce_23,
 1, 30, :_reduce_24,
 1, 26, :_reduce_25,
 0, 26, :_reduce_none,
 1, 35, :_reduce_27,
 2, 35, :_reduce_28,
 2, 35, :_reduce_29,
 2, 35, :_reduce_30,
 1, 36, :_reduce_none,
 1, 36, :_reduce_32,
 2, 36, :_reduce_33,
 1, 36, :_reduce_34,
 1, 32, :_reduce_35,
 2, 32, :_reduce_36 ]

racc_reduce_n = 37

racc_shift_n = 59

racc_action_table = [
    20,    54,    20,    20,    20,    20,    19,    39,    19,    19,
    19,    19,    41,    42,    43,    44,    46,    37,    20,    20,
    20,    20,    20,    40,    19,    19,    19,    19,    19,    13,
    37,    37,    37,    17,     9,    10,    11,    20,    14,    15,
    20,    20,    20,    19,    25,    18,    19,    19,    19,    53,
     1,    32,    33,    29,    50,    32,    33,    29,    32,    33,
    29,    32,    33,    29,    56,     8,     7,     5,     4,    58 ]

racc_action_check = [
    27,    35,    35,    33,    10,    51,    27,    24,    35,    33,
    10,    51,    27,    27,    27,    27,    27,    51,    52,    23,
    48,     9,    13,    26,    52,    23,    48,     9,    13,     6,
    52,    23,    48,     6,     6,     6,     6,    29,     6,     6,
    44,    32,    17,    29,    11,     7,    44,    32,    17,    34,
     0,    34,    34,    34,    30,    30,    30,    30,    15,    15,
    15,    14,    14,    14,    36,     5,     3,     2,     1,    55 ]

racc_action_pointer = [
    48,    63,    67,    60,   nil,    65,    26,    40,   nil,    16,
    -1,    39,   nil,    17,    47,    44,   nil,    37,   nil,   nil,
   nil,   nil,   nil,    14,     2,   nil,    19,    -5,   nil,    32,
    41,   nil,    36,    -2,    37,    -3,    53,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,    35,   nil,   nil,   nil,    15,   nil,
   nil,     0,    13,   nil,   nil,    58,   nil,   nil,   nil ]

racc_action_default = [
   -37,   -37,   -37,    -4,    -2,   -37,   -37,   -37,    59,   -37,
   -37,   -37,    -5,   -26,   -37,   -37,    -7,   -37,    -3,   -24,
   -23,    -8,   -20,    -9,   -10,   -35,   -37,   -25,   -27,   -37,
   -37,   -15,   -37,   -37,   -37,   -37,   -37,   -22,   -21,   -36,
    -1,   -32,   -29,   -30,   -37,   -28,   -34,   -31,   -19,   -16,
   -13,   -17,   -18,   -14,    -6,   -37,   -11,   -33,   -12 ]

racc_goto_table = [
    21,    23,    49,     2,    28,    35,    49,    16,    36,    30,
    34,    12,    26,    24,    38,     6,     3,    27,    47,    45,
    48,   nil,   nil,    51,    52,   nil,    55,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,    57,   nil,   nil,   nil,    38,
   nil,   nil,    38,    38 ]

racc_goto_check = [
     8,     9,    12,     1,     8,     6,    12,     7,     8,    11,
    11,     5,     4,    10,     8,     3,     2,    13,     8,    14,
     9,   nil,   nil,     9,     9,   nil,     8,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,     8,   nil,   nil,   nil,     8,
   nil,   nil,     8,     8 ]

racc_goto_pointer = [
   nil,     3,    15,    12,    -1,     5,   -12,     1,    -9,    -9,
     2,    -5,   -28,     4,    -8 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,    22,   nil,
   nil,   nil,    31,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :XCLASS => 2,
 :XRULE => 3,
 :XEND => 4,
 :XSYMBOL => 5,
 "<" => 6,
 :XCONV => 7,
 :XSTART => 8,
 :XTOKEN => 9,
 :XOPTION => 10,
 :STRING => 11,
 :XPRECHIGH => 12,
 :XPRECLOW => 13,
 :XLEFT => 14,
 :XRIGHT => 15,
 :XNONASSOC => 16,
 "|" => 17,
 ";" => 18,
 ":" => 19,
 "=" => 20,
 :ACTION => 21 }

racc_use_result_var = true

racc_nt_base = 22

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n,
 racc_use_result_var ]

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

module_eval <<'.,.,', '(build)', 83
  def _reduce_1( val, _values, result )
      @ruletable.end_register_rule
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 87
  def _reduce_2( val, _values, result )
      @class_name = val[0].id2name
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 90
  def _reduce_3( val, _values, result )
      @super_class = val[2].id2name
   result
  end
.,.,

 # reduce 4 omitted

 # reduce 5 omitted

module_eval <<'.,.,', '(build)', 100
  def _reduce_6( val, _values, result )
      @symboltable.end_register_conv
   result
  end
.,.,

 # reduce 7 omitted

module_eval <<'.,.,', '(build)', 106
  def _reduce_8( val, _values, result )
      @ruletable.register_start val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 110
  def _reduce_9( val, _values, result )
      @symboltable.register_token val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 114
  def _reduce_10( val, _values, result )
      val[1].each do |s|
        @ruletable.register_option s.to_s
      end
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 121
  def _reduce_11( val, _values, result )
      @symboltable.register_conv( val[0], val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 125
  def _reduce_12( val, _values, result )
      @symboltable.register_conv( val[1], val[2] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 130
  def _reduce_13( val, _values, result )
      @symboltable.end_register_prec( true )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 134
  def _reduce_14( val, _values, result )
      @symboltable.end_register_prec( false )
   result
  end
.,.,

 # reduce 15 omitted

 # reduce 16 omitted

module_eval <<'.,.,', '(build)', 144
  def _reduce_17( val, _values, result )
      @symboltable.register_prec( :Left, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 148
  def _reduce_18( val, _values, result )
      @symboltable.register_prec( :Right, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 152
  def _reduce_19( val, _values, result )
      @symboltable.register_prec( :Nonassoc, val[1] )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 157
  def _reduce_20( val, _values, result )
      result = val
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 161
  def _reduce_21( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 # reduce 22 omitted

module_eval <<'.,.,', '(build)', 167
  def _reduce_23( val, _values, result )
      result = @symboltable.get( result )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 171
  def _reduce_24( val, _values, result )
      result = @symboltable.get( eval '"' + val[0] + '"' )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 176
  def _reduce_25( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
   result
  end
.,.,

 # reduce 26 omitted

module_eval <<'.,.,', '(build)', 185
  def _reduce_27( val, _values, result )
      result = val
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 189
  def _reduce_28( val, _values, result )
      result.push val[1]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 193
  def _reduce_29( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result.clear
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 200
  def _reduce_30( val, _values, result )
      pre = result.pop
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result = [pre]
   result
  end
.,.,

 # reduce 31 omitted

module_eval <<'.,.,', '(build)', 211
  def _reduce_32( val, _values, result )
      result = OrMark.new( @scanner.lineno )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 215
  def _reduce_33( val, _values, result )
      result = Prec.new( val[1], @scanner.lineno )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 219
  def _reduce_34( val, _values, result )
      result = UserAction.new( *result )
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 223
  def _reduce_35( val, _values, result )
      result = [ result.id2name ]
   result
  end
.,.,

module_eval <<'.,.,', '(build)', 226
  def _reduce_36( val, _values, result )
      result.push val[1].id2name
   result
  end
.,.,

 def _reduce_none( val, _values, result )
  result
 end

  end   # class RaccParser

end   # module Racc
